%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: December 2016
%
%   References:
%   N. Alevizos, X. Fu, N. Sidiropoulos, Y. Yang, A. Bletsas, 
%    "Non-uniform Directional Dictionary-Based Limited Feedback for 
%    Massive MIMO Systems."
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**


clear all;
close all;
clc;

warning ('off','all');




%maximum number of iteration
Max_iter =  500;

%% Tx/Rx other parameters
%number of training symbols
N_tr =  64;
%number of transmit antennas
M_tx_all = [64: 64 : 512];
% M_tx_all = [512]
%number of receive antennas
M_rx = 1;
%normalized antenna spacing
D_y = 1/2;
%number of quantization bits per dimension
B_q_all = [1 : 1 : 2] ;



%% large scale fading parameters
%Carrier frequency
Carrier_Freq=2*10^9;
% % Wavelength
lambda=3*10^8/Carrier_Freq;
%reference distance
d0 = 1;
%transmit power
P_tx_avg = 0.5;


%% Noise calculations
% Noise power  
N0=10^(-10);

 
%%
avgSNR = zeros(1, length(M_tx_all));

%% BFG
BFGain_LS_SQ = zeros(length(B_q_all), length(M_tx_all));
BFGain_LS_VQ = zeros(length(B_q_all), length(M_tx_all));
BFGain_perfect_CSI = zeros(length(B_q_all), length(M_tx_all));


for m_tx_iter = 1 : length(M_tx_all) % 
    
    M_tx = M_tx_all(m_tx_iter);
    
    
    %3GPP parameters
    phi_3dB = 55 * pi / 180;
    Am = 30;
    Ant_Gain_dB =  6.7849;
    
    
    
    for B_q_count = 1 : length(B_q_all) % 
        
        % 
        B_q = B_q_all(B_q_count);
        
        for iter = 1 : Max_iter  %number of experiments counter
%             iter
            
            %number of paths
            L_p = randi([5,20]) ;
            
            
            %steering vectors of transmitter and receiver
            phi_tx = -pi/2 +  pi * rand(L_p, 1);
            %impact of antenna directivity pattern
            c_f_paths =  10^(Ant_Gain_dB/20) *  max(10.^( -12 * (1/20) * (phi_tx/phi_3dB).^2),10^( -Am/20 ) );
            c_f_paths_kron = kron(c_f_paths.',ones(M_tx,1));
            phi_rx =  -pi/2 + pi *  rand(L_p, 1);
            
                        %creating steering matrices (for ULA)
            A_tx = (1/sqrt(M_tx))*    c_f_paths_kron .* exp(-1i*2*pi*D_y* kron([0 : M_tx - 1].' , sin(phi_tx).'));
            A_rx = (1/sqrt(M_rx)) * exp(-1i*2*pi*D_y* kron([0 : M_rx - 1].' , sin(phi_rx).'));
            
            
               
            %% large scale fading parameters
            
            %Carrier frequency
            Carrier_Freq=2*10^9;
            % % Wavelength
            lambda=3*10^8/Carrier_Freq;
            % Pathloss exponent
            n_pathloss = 2.8 + 0.1*randn;
            
            %distance between BS (metters)
            BS_UE_dist = 80 + 40 * rand(L_p,1);
            %             BS_UE_dist = 100;
            %path loss
            ro=((lambda/(4*pi*d0))^2).*(d0./BS_UE_dist).^n_pathloss;
            
            %shadowing standard deviation 4 dB
            shadowing_std = 4;
            %noise due to shadowing
            loss_dB = 10*log10(ro) + shadowing_std * randn(L_p,1);
            %average received power including shadowing
            shadowing = 10.^(0.1*loss_dB);
            
            
            
            
            %% small scale fading parameters
            %std of channel
            sigma_h = sqrt(shadowing);
            
            %Rician K factor
            K_p = 50*rand(L_p, 1);
            
            %channel vector
            chan = (sqrt(M_tx*M_rx)/sqrt(L_p)) * ...
                (exp(-1i*2*pi*rand(L_p,1)).*sqrt(K_p ./ (K_p + 1)) .* sigma_h +  (sigma_h .* sqrt(0.5 ./ (K_p + 1) ) .* (randn(L_p, 1) + 1i*randn(L_p, 1)))) ;
            
            %double directional channel
            H = A_rx * diag(chan) * A_tx';
            
             %training signal
            tmp = (1/sqrt(2*M_tx)) * (sign(randn(max(M_tx,N_tr))) + 1i*sign(randn(max(M_tx,N_tr))));
            S_tr = tmp(1 : M_tx, 1 : N_tr);
            
            %noise vector
            W =   sqrt(N0/2) * (randn(M_rx, N_tr) + 1i*randn(M_rx, N_tr));
            
            %received signal  
            Y =  sqrt(P_tx_avg) * H * S_tr + W;
            
            
            %vectorized y
            y =  Y(:);
            
              %beamforming gain of perfect CSI
            BFGain_perfect_CSI(B_q_count, m_tx_iter)  = BFGain_perfect_CSI(B_q_count, m_tx_iter) + norm(sqrt(P_tx_avg)*H, 'fro')^2;
            
            
          
          
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% LS
            %classic LS
            H_LS = Y*pinv(S_tr);
            
            %scalar quantization
            [ H_LS_1 ] = Quantize_Lloyd( H_LS, B_q );
            BFGain_LS_SQ(B_q_count, m_tx_iter) = BFGain_LS_SQ(B_q_count, m_tx_iter) + (sqrt(P_tx_avg)*abs(H * H_LS_1')/norm(H_LS_1,'fro'))^2;
            
            
            %vector quantization
            [ H_LS_VQ ] = M_PSK_sequence_detection_faster2(  2^B_q, H_LS );
            BFGain_LS_VQ(B_q_count, m_tx_iter) =  BFGain_LS_VQ(B_q_count, m_tx_iter) + (sqrt(P_tx_avg)*abs(H * H_LS_VQ')/norm(H_LS_VQ,'fro'))^2;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
            
            
            
        end %end  
        
        %obtain the averages
        BFGain_perfect_CSI(B_q_count, m_tx_iter) =  BFGain_perfect_CSI(B_q_count, m_tx_iter) / Max_iter
        BFGain_LS_SQ(B_q_count, m_tx_iter)  =  BFGain_LS_SQ(B_q_count, m_tx_iter) / Max_iter
        BFGain_LS_VQ(B_q_count, m_tx_iter) =  BFGain_LS_VQ(B_q_count, m_tx_iter)/Max_iter

    end %end  

end %end


