%%  Author: Panos N. Alevizos
%
%   Ph.D. Student and Research Assistant
%   School of Electrical and Computer Engineering
%   Technical University of Crete
%   Email: palevizos@isc.tuc.gr or bigpan88@hotmail.com
%   Webpage: http://users.isc.tuc.gr/~palevizos/
%
%   Date: December 2016
%
%   References:
%   N. Alevizos, X. Fu, N. Sidiropoulos, Y. Yang, A. Bletsas,
%    "Non-uniform Directional Dictionary-Based Limited Feedback for
%    Massive MIMO Systems."
%
%   **Inquiries regarding the script provided below are cordially welcome.
%   In case you spot a bug, please let me know.
%   If you use some piece of code for your own work, please cite the
%   article above.**



clear all;
close all;
clc;

warning ('off','all');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%maximum number of iterations
Max_iter = 50;

%% Tx/Rx other parameters
%number of training symbols
N_tr =  64;
%number of transmit antennas
M_tx_all = [64: 64 : 512];
%number of receive antennas
M_rx = 1;
%normalized antenna spacing
D_y = 1/2;
%number of quantization bits per dimension
B_q_all = [1] ;
%


%% Noise calculation
% Noise power
N0=10^(-10);


%% BFG
BFGain_CS1 = zeros(length(B_q_all), length(M_tx_all));
BFGain_perfect_CSI = zeros(length(B_q_all), length(M_tx_all));

%% large scale fading parameters
%Carrier frequency
Carrier_Freq=2*10^9;
%Propagation Wavelength
lambda=3*10^8/Carrier_Freq;
%reference distance
d0 = 1;
% P_tx_avg
P_tx_avg = 0.5;

%dipole directivity pattern parameters
G_m = 2.35618;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% measure the power of the channel
R2 = zeros(1, length(M_tx_all));
for m_tx_iter = 1 : length(M_tx_all) %
    
    M_tx = M_tx_all(m_tx_iter);
    
    for iter = 1 : 1000
        %number of paths
        L_p = randi([5,20]) ;
        
        
        %steering vectors of transmitter and receiver
        phi_tx = -pi/2 +  pi * rand(L_p, 1);
        %impact of antenna directivity pattern
        c_f_paths =  G_m * cos(phi_tx).^3;
        c_f_paths_kron = kron(c_f_paths.',ones(M_tx,1));
        phi_rx =  -pi/2 + pi *  rand(L_p, 1);
        
        %creating steering matrices (for ULA)
        A_tx = (1/sqrt(M_tx))* c_f_paths_kron .* exp(-1i*2*pi*D_y* kron([0 : M_tx - 1].' , sin(phi_tx).'));
        A_rx = (1/sqrt(M_rx)) * exp(-1i*2*pi*D_y* kron([0 : M_rx - 1].' , sin(phi_rx).'));
        
        
        
        
        %% large scale fading parameters
        
        % Pathloss exponent
        n_pathloss = 2.8 + 0.1*randn;
        
        %distance between BS (metters)
        BS_UE_dist = 80 + 40 * rand(L_p,1);
        %             BS_UE_dist = 100;
        %path loss
        ro=((lambda/(4*pi*d0))^2).*(d0./BS_UE_dist).^n_pathloss;
        
        %shadowing standard deviation 4 dB
        shadowing_std = 4;
        %noise due to shadowing
        loss_dB = 10*log10(ro) + shadowing_std * randn(L_p,1);
        %average received power including shadowing
        shadowing = 10.^(0.1*loss_dB);
        
        
        
        
        %% small scale fading parameters
        %std of channel
        sigma_h = sqrt(shadowing);
        
        %Rician K factor
        K_p = 50*rand(L_p, 1);
        
        %channel vector
        chan = (sqrt(M_tx*M_rx)/sqrt(L_p)) * ...
            (exp(-1i*2*pi*rand(L_p,1)).*sqrt(K_p ./ (K_p + 1)) .* sigma_h +  (sigma_h .* sqrt(0.5 ./ (K_p + 1) ) .* (randn(L_p, 1) + 1i*randn(L_p, 1)))) ;
        
        %double directional channel
        H = A_rx * diag(chan) * A_tx';
        
        
        
        R2(m_tx_iter) = R2(m_tx_iter) + norm(H, 'fro');
    end
    R2(m_tx_iter)  = R2(m_tx_iter)  / 1000;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%v1
norm_x = 0.01*R2;
cons_C = 0.042 ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for m_tx_iter = 1 : length(M_tx_all) %
    
    M_tx = M_tx_all(m_tx_iter);
    
    %% create dictionary
    %dipole parameters
    G_tx =  180;
    G_rx = 180;
    G_m = 2.35618;
    
    
    %dictionary at BS accounting antenna directivity pattern
    phi_uni_tx = -pi/2 + (pi/(G_tx+1)):  (pi/(G_tx+1)) : pi/2 - (pi/(G_tx+1));
    cummulative_antenna_pattern_tx = c_F_dipole( phi_uni_tx, G_m, -pi/2);
    y1 = cummulative_antenna_pattern_tx(1);
    yend = cummulative_antenna_pattern_tx(end);
    y_length = yend - y1;
    y_axis = y1 + y_length/(G_tx+1):   y_length/(G_tx+1) : yend - y_length/(G_tx+1) ;
    phi_dict_tx = c_F_dipole_inv_v2(   y_axis , G_m, -pi/2);
    c_f_dictionary_tx = G_m * cos(phi_dict_tx).^3 ;
    c_f_kron_dictionary_tx = kron(c_f_dictionary_tx,ones(M_tx,1));
    
    phi_dict_rx = -pi/2 + (pi/(G_rx+1)):  (pi/(G_rx+1)) : pi/2 - (pi/(G_rx+1));
    
    A_d_tx =  (1/sqrt(M_tx))* c_f_kron_dictionary_tx.* exp(-1i*2*pi*D_y* kron([0 : M_tx - 1].' , sin(phi_dict_tx)));
    A_d_rx =  (1/sqrt(M_rx)) *  exp(-1i*2*pi*D_y* kron([0 : M_rx - 1].' , sin(phi_dict_rx)));
    
    
     
    for B_q_count = 1 : length(B_q_all) %
        
        %
        B_q = B_q_all(B_q_count);
        
        for iter = 1 : Max_iter  %number of experiments counter
            %             iter
            
            
            %number of paths
            L_p = randi([5,20]) ;
            
            
            %steering vectors of transmitter and receiver
            phi_tx = -pi/2 +  pi * rand(L_p, 1);
            %impact of antenna directivity pattern
            c_f_paths =  G_m * cos(phi_tx).^3;
            c_f_paths_kron = kron(c_f_paths.',ones(M_tx,1));
            phi_rx =  -pi/2 + pi *  rand(L_p, 1);
            
            %creating steering matrices (for ULA)
            A_tx = (1/sqrt(M_tx))* c_f_paths_kron .* exp(-1i*2*pi*D_y* kron([0 : M_tx - 1].' , sin(phi_tx).'));
            A_rx = (1/sqrt(M_rx)) * exp(-1i*2*pi*D_y* kron([0 : M_rx - 1].' , sin(phi_rx).'));
            
            
            
            
            %% large scale fading parameters
            
            % Pathloss exponent
            n_pathloss = 2.8 + 0.1*randn;
            
            %distance between BS (metters)
            BS_UE_dist = 80 + 40 * rand(L_p,1);
            %             BS_UE_dist = 100;
            %path loss
            ro=((lambda/(4*pi*d0))^2).*(d0./BS_UE_dist).^n_pathloss;
            
            %shadowing standard deviation 4 dB
            shadowing_std = 4;
            %noise due to shadowing
            loss_dB = 10*log10(ro) + shadowing_std * randn(L_p,1);
            %average received power including shadowing
            shadowing = 10.^(0.1*loss_dB);
            
            
            
            
            %% small scale fading parameters
            %std of channel
            sigma_h = sqrt(shadowing);
            
            %Rician K factor
            K_p = 50*rand(L_p, 1);
            
            %channel vector
            chan = (sqrt(M_tx*M_rx)/sqrt(L_p)) * ...
                (exp(-1i*2*pi*rand(L_p,1)).*sqrt(K_p ./ (K_p + 1)) .* sigma_h +  (sigma_h .* sqrt(0.5 ./ (K_p + 1) ) .* (randn(L_p, 1) + 1i*randn(L_p, 1)))) ;
            
            %double directional channel
            H = A_rx * diag(chan) * A_tx';
            
            
            
            
            %training signal
            tmp = (1/sqrt(2*M_tx)) * (sign(randn(max(M_tx,N_tr))) + 1i*sign(randn(max(M_tx,N_tr))));
            S_tr = tmp(1 : M_tx, 1 : N_tr);
            
            %noise vector
            W =   sqrt(N0/2) * (randn(M_rx, N_tr) + 1i*randn(M_rx, N_tr));
            
            %received signal
            Y =  sqrt(P_tx_avg) * H * S_tr + W;
            
            %vectorized y
            y =  Y(:);
            
            
            %measurement matrix
            Q = kron((S_tr.')*conj(A_d_tx), A_d_rx);
            %         X = kron((S_tr.')*conj(A_tx), A_rx);
            [numX_rows, numX_cols] = size(Q);
            
            
            
            
            
            
            
            %%
            %the number of columns of P
            N_fb = N_tr*M_rx ;
            
            %matrix P
            P_tmp =(1/sqrt(M_rx*N_tr)) * dftmtx(M_rx*N_tr);
            P = P_tmp(:, randperm(M_rx*N_tr,N_fb));
            
            
            %QAM quantization
            b_complex = sign(real(P' * y)) +  1i*sign(imag(P' * y));
            
            %
            C_complex = Q'*P;
            
            b = [real(b_complex); imag(b_complex)];
            C_real = [real(C_complex) ; imag(C_complex)];
            C_imag = [-imag(C_complex) ;  real(C_complex)];
            C = [C_real, C_imag];
            
            
            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %% CS approach
            gamma = cons_C  * sqrt( log(G_rx * G_tx)/ (2 * N_fb) ) * 2 * N_fb;
            a0 =   C * b;
            a1 =   sign(a0) .* (abs(a0) - gamma) .* (abs(a0) >= gamma);
            x_est_CS1 = (max(abs(a0)) >= gamma)* norm_x(m_tx_iter) * a1 / norm(a1);
            g_CS1_complex = (x_est_CS1(1:end/2) + 1i * x_est_CS1(end/2 + 1 : end));         %
            H_est_CS1 =    A_d_rx * reshape(g_CS1_complex, G_rx, G_tx ) * A_d_tx';
            if norm(H_est_CS1,'fro') > 0
                BFGain_CS1(B_q_count, m_tx_iter) =  BFGain_CS1(B_q_count, m_tx_iter)+ (sqrt(P_tx_avg)*abs(H * H_est_CS1')/norm(H_est_CS1,'fro'))^2;
            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
        end %end
        
        %obtain the averages
        BFGain_CS1(B_q_count, m_tx_iter) =  BFGain_CS1(B_q_count, m_tx_iter)/ Max_iter
        
    end %end
    
end %end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
